package be.iminds.ilabt.jfed.ui.cli2.instruction;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Arrays;

/**
 * ActionOutput: instructions on how to perform output for the action
 */
public class ActionOutput {
    public enum Format {
        TXT("text"), JSON(null);
        @Nullable private final String altname;

        Format(@Nullable String altname) {
            this.altname = altname;
        }

        public boolean matches(@Nonnull String value) {
            if (value.equalsIgnoreCase(name()))
                return true;
            return value.equalsIgnoreCase(altname);
        }

        @JsonCreator
        public static Format parse(@Nonnull String value) {
            return Arrays.stream(Format.values())
                    .filter(m -> m.matches(value))
                    .findAny()
                    .orElseThrow(() -> new IllegalArgumentException("Format not known: "+value));
        }
    }

    @Nonnull public final Format format;
    @Nonnull public final OutputTarget target;
    @Nullable public final String filename;

    public ActionOutput(@JsonProperty("format") @Nullable Format format,
                        @JsonProperty("target") @Nullable OutputTarget target,
                        @JsonProperty("filename") @Nullable String filename) {
        this.format = format == null ? new ActionOutput().format : format;
        this.target = target == null ? new ActionOutput().target : target;
        this.filename = filename;
    }

    public ActionOutput() {
        this.format = Format.JSON;
        this.target = OutputTarget.STDOUT;
        this.filename = null;
    }

    @Nonnull
    public Format getFormat() {
        return format;
    }

    @Nonnull
    public OutputTarget getTarget() {
        return target;
    }

    @Nullable
    public String getFilename() {
        return filename;
    }
}
